//Compiled in Processing 3.2, Oct 2017

// Layout variables 
float constantHorizontalMargin = 160;
float constantVerticalMargin = 40;

// width and height for each cell
float cellWidth, cellHeight;

// 2D matrix table related variables
Table table, table2;
int numRows, numColumns;
float[][] dataMatrix;
float maxValue, minValue;

int order = 0;

PImage colorBar;

String[] colorPath = {"bwReBlend129b.jpg", "bwReBlend129d.jpg", "bwReBlend129h.jpg", "Colormap4.jpg"};

boolean colorScheme = false;

void setup(){
    size(1920, 1080); // setup the size of the window
  // In Processing 3, the size() function must be the first line of code inside setup()
  // and needs to use numbers instead of variables. 

  // Make the screen resizable.                     
  surface.setResizable(true);
  
  //Put data into Processing table from csv
  table = loadTable("item_month.csv", "header");
  numRows = table.getRowCount(); 
  numColumns = table.getColumnCount();
  dataMatrix = new float[numRows][numColumns-3];
  println("Rows: " + numRows + " Columns: " + numColumns);
  
  for(int i=0; i<numRows; i++){
    for(int j=0; j<numColumns-3; j++){
      dataMatrix[i][j] = table.getFloat(i,j+3);
     
      //Find the max value of the whole data
      if(dataMatrix[i][j] > maxValue){
        maxValue = dataMatrix[i][j];
      }
    }
  }
  
  minValue = maxValue;
  for(int i=0; i<numRows; i++){
    for(int j=0; j<numColumns-3; j++){
      if(dataMatrix[i][j] < minValue){
        minValue = dataMatrix[i][j];
      }
    }
  }
  
  println("Max Value is " + maxValue);
  println("Min Value is " + minValue);
  
  table2 = loadTable("item_month.csv", "header");
  table2.sort(0);
}

void draw(){
  background(250);
  //translate(120, 0);
  
  //modifying the cell size according to window size
  cellWidth = (width - constantHorizontalMargin*2)/(numColumns-3);
  cellHeight = (height - constantVerticalMargin*2)/numRows;
  
  //Draw 2D Matrix
  colorMode(HSB, 360, 100, 100);
  for(int i=0; i<numRows; i++){
    for(int j=0; j<numColumns-3; j++){
      stroke(250);
      strokeWeight(1);
      float greyScale = map(dataMatrix[i][j], minValue, maxValue, 0, 100);
      if(order == 0){
        if(table.getInt(i, 0) == 2479267) {
          fill(204, greyScale, 100);
        }
        else if(table.getInt(i, 0) == 1943806){
          fill(144, greyScale, 98);
        }
        else if(table.getInt(i, 0) == 2132112){
          fill(46, greyScale, 96);
        } 
        else if(table.getInt(i, 0) == 3284034){
          fill(4, greyScale, 94);
        }
      }
      else if(order == 1){
        if(table2.getInt(i, 0) == 2479267) {
          fill(204, greyScale, 100);
        }
        else if(table2.getInt(i, 0) == 1943806){
          fill(144, greyScale, 98);
        }
        else if(table2.getInt(i, 0) == 2132112){
          fill(46, greyScale, 96);
        } 
        else if(table2.getInt(i, 0) == 3284034){
          fill(4, greyScale, 94);
        }
      }
      rect(constantHorizontalMargin + cellWidth*j, constantVerticalMargin 
           + cellHeight*i, cellWidth, cellHeight);
    }
  }
  
  colorMode(RGB, 255, 255, 255);
  //Draw horizontal labels
  for(int i=0; i<12; i++){
    int year = 2006;
    textSize(10);
    textAlign(CENTER, TOP);
    fill(0);
    text(year+i, constantHorizontalMargin + cellWidth*(6+12*i), 
         constantVerticalMargin +cellHeight*numRows + 5);
         
    //draw the seperation lines
    if(i != 0){
      stroke(#009EE5);
      line(constantHorizontalMargin + cellWidth*(12*i), constantVerticalMargin +cellHeight*numRows + 3,  
           constantHorizontalMargin + cellWidth*(12*i), constantVerticalMargin +cellHeight*numRows + 20);
    }
  }
   
  //Draw vertical labels
  for(int i=0; i<numRows; i++){
    textAlign(LEFT, CENTER);
    fill(0);
    textSize(10);
    if(order == 0) text(table.getString(i, 0).toLowerCase() + "-" + table.getString(i, 1).toLowerCase(), constantHorizontalMargin-100, constantVerticalMargin +cellHeight*(0.5+i));
    else if(order == 1) text(table2.getString(i, 0).toLowerCase() + "-" + table2.getString(i, 1).toLowerCase(), constantHorizontalMargin-100, constantVerticalMargin +cellHeight*(0.5+i));
    
     //show total number on the right side
    textAlign(LEFT, CENTER);
    if(order == 0) text(table.getInt(i,2),width-constantHorizontalMargin+6,constantVerticalMargin +cellHeight*(0.5+i));
    else if(order == 1) text(table2.getInt(i,2),width-constantHorizontalMargin+6,constantVerticalMargin +cellHeight*(0.5+i)); 
  }
  
  //Draw Title
  textAlign(CENTER, CENTER);
  textSize(16);
  text("Blade Runner Monthly Preformance from 2006 to 2017", width/2, constantVerticalMargin/2);
  
  if(keyPressed){
    if(key == '1'){
      for(int i=0; i<numRows; i++){
        for(int j=0; j<numColumns-3; j++){
          dataMatrix[i][j] = table.getFloat(i,j+3);
        }
      }    
      order = 0;
    }
  
    else if(key == '2'){
      for(int i=0; i<numRows; i++){
        for(int j=0; j<numColumns-3; j++){
          dataMatrix[i][j] = table2.getFloat(i,j+3);
        }
      }
      order = 1;
    }
    
    else if(key == 's'){
      saveFrame();
    }
  }
}